-- Create free recipe for each fluid.
for _, fluid in pairs(data.raw["fluid"]) do
	data:extend(
	{
		{
			type = "recipe",
			name = creative_mode_defines.names.free_fluid_recipe_prefix .. fluid.name,
			category = creative_mode_defines.names.recipe_categories.free_fluids,
			ingredients = {},
			results =
			{
				{type = "fluid", name = fluid.name, amount = 50}
			},
			main_product = fluid.name,
			subgroup = creative_mode_defines.names.item_subgroups.free_fluids,
			enabled = false
		}
	})
end

-- Some mods (e.g. Bob's Modules) add limitations to all productivity modules. But we want our super productivity module to be limitless.
data.raw["module"][creative_mode_defines.names.recipes.super_productivity_module].limitation = nil

--------------------------------------------------------------

-- All possible data raw names for items that have place results of entities.
local possible_data_raw_types_for_items_of_entities =
{
	"item",
	"item-with-entity-data",	-- 0.13
	"item-with-label",			-- 0.13
}
-- Make a dictionary for recording all items' place results, so we can find whether the items for the enemy worms and spawners already exist.
local item_place_results = {}
for _, raw_name in pairs(possible_data_raw_types_for_items_of_entities) do
	local data_raw = data.raw[raw_name]
	if data_raw then
		for _, item in pairs(data_raw) do
			if item.place_result then
				item_place_results[item.place_result] = true
			end
		end
	end
end

-- Returns whether there is already an item for the given entity.
local function has_item_for_enemy_entity(entity)
	return item_place_results[entity.name] == true
end

-- Creates item and free recipe for each "worm", and "spawner".
local function create_item_and_recipe_for_enemies_in_data_raw(raw_name)
	for _, entity in pairs(data.raw[raw_name]) do
		-- Add item and recipe only if there is no item for the entity.
		-- If there is already an item, there is high chance that its recipe is also there.
		if not has_item_for_enemy_entity(entity) then
			local item_name = creative_mode_defines.names.enemy_item_prefix .. entity.name
			local recipe_name = creative_mode_defines.names.enemy_recipe_prefix .. entity.name
			data:extend(
			{
				{
					type = "item",
					name = item_name,
					icon = entity.icon,
					flags = {"goes-to-quickbar", "hidden"},
					subgroup = creative_mode_defines.names.item_subgroups.enemies,
					--order = entity.order,
					place_result = entity.name,
					stack_size = 50
				},
				{
					type = "recipe",
					name = recipe_name,
					category = creative_mode_defines.names.recipe_categories.enemies,
					ingredients = {},
					result = item_name,
					enabled = false
				}
			})
		end
	end
end
-- Worms.
create_item_and_recipe_for_enemies_in_data_raw("turret")
-- Spawners.
create_item_and_recipe_for_enemies_in_data_raw("unit-spawner")


--[[
-- Returns whether the given item has the "hidden" flag.
local function has_hidden_flag(item)
	if not item.flags then
		return false
	end
	for _, flag in pairs(item.flags) do
		if flag == "hidden" then
			return true
		end
	end
	return false
end

-- Counts the items in data raw of the given name.
local function count_items(raw_name)
	local count = 0
	for _, item in pairs(data.raw[raw_name]) do
		-- Skip the hidden items. They are hidden for good reason.
		if creative_mode_config.creative_provider_chest.contain_hidden_items or not has_hidden_flag(item) then
			count = count + 1
		end
	end
	return count
end
--]]